@@ -17,13 +17,13 @@
 %                                October 2002                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -89,6 +89,7 @@
 #include "magick/segment.h"
 #include "magick/splay-tree.h"
 #include "magick/string_.h"
+#include "magick/string-private.h"
 #include "magick/thread-private.h"
 #include "magick/threshold.h"
 #include "magick/transform.h"
@@ -122,12 +123,271 @@
 %    o exception: return any errors or warnings in this structure.
 %
 */
+
+typedef struct _EdgeInfo
+{
+  double
+    left,
+    right,
+    top,
+    bottom;
+} EdgeInfo;
+
+static double GetEdgeBackgroundFactor(const Image *image,
+  const CacheView *image_view,const GravityType gravity,const size_t width,
+  const size_t height,const ssize_t x_offset,const ssize_t y_offset,
+  ExceptionInfo *exception)
+{
+  CacheView
+    *edge_view;
+
+  double
+    factor;
+
+  Image
+    *edge_image;
+
+  MagickPixelPacket
+    background,
+    pixel;
+
+  RectangleInfo
+    edge_geometry;
+
+  register const PixelPacket
+    *p;
+
+  ssize_t
+    y;
+
+  /*
+    Determine the percent of image background for this edge.
+  */
+  switch (gravity)
+  {
+    case NorthWestGravity:
+    case NorthGravity:
+    default:
+    {
+      p=GetCacheViewVirtualPixels(image_view,0,0,1,1,exception);
+      break;
+    }
+    case NorthEastGravity:
+    case EastGravity:
+    {
+      p=GetCacheViewVirtualPixels(image_view,(ssize_t) image->columns-1,0,1,1,
+        exception);
+      break;
+    }
+    case SouthEastGravity:
+    case SouthGravity:
+    {
+      p=GetCacheViewVirtualPixels(image_view,(ssize_t) image->columns-1,
+        (ssize_t) image->rows-1,1,1,exception);
+      break;
+    }
+    case SouthWestGravity:
+    case WestGravity:
+    {
+      p=GetCacheViewVirtualPixels(image_view,0,(ssize_t) image->rows-1,1,1,
+        exception);
+      break;
+    }
+  }
+  GetMagickPixelPacket(image,&background);
+  SetMagickPixelPacket(image,p,(IndexPacket *) NULL,&background);
+  edge_geometry.width=width;
+  edge_geometry.height=height;
+  edge_geometry.x=x_offset;
+  edge_geometry.y=y_offset;
+  GravityAdjustGeometry(image->columns,image->rows,gravity,&edge_geometry);
+  edge_image=CropImage(image,&edge_geometry,exception);
+  if (edge_image == (Image *) NULL)
+    return(0.0);
+  factor=0.0;
+  GetMagickPixelPacket(edge_image,&pixel);
+  edge_view=AcquireVirtualCacheView(edge_image,exception);
+  for (y=0; y < (ssize_t) edge_image->rows; y++)
+  {
+    register ssize_t
+      x;
+
+    p=GetCacheViewVirtualPixels(edge_view,0,y,edge_image->columns,1,exception);
+    if (p == (const PixelPacket *) NULL)
+      break;
+    for (x=0; x < (ssize_t) edge_image->columns; x++)
+    {
+      SetMagickPixelPacket(edge_image,p,(IndexPacket *) NULL,&pixel);
+      if (IsMagickColorSimilar(&pixel,&background) == MagickFalse)
+        factor++;
+      p++;
+    }
+  }
+  factor/=((double) edge_image->columns*edge_image->rows);
+  edge_view=DestroyCacheView(edge_view);
+  edge_image=DestroyImage(edge_image);
+  return(factor);
+}
+
+static inline double GetMinEdgeBackgroundFactor(const EdgeInfo *edge)
+{
+  double
+    factor;
+
+  factor=MagickMin(MagickMin(MagickMin(edge->left,edge->right),edge->top),
+    edge->bottom);
+  return(factor);
+}
+
+static RectangleInfo GetEdgeBoundingBox(const Image *image,
+  ExceptionInfo *exception)
+{
+  CacheView
+    *edge_view;
+
+  const char
+    *artifact;
+
+  double
+    background_factor,
+    percent_background;
+
+  EdgeInfo
+    edge,
+    vertex;
+
+  Image
+    *edge_image;
+
+  RectangleInfo
+    bounds;
+
+  /*
+    Get the image bounding box.
+  */
+  assert(image != (Image *) NULL);
+  assert(image->signature == MagickCoreSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  SetGeometry(image,&bounds);
+  edge_image=CloneImage(image,0,0,MagickTrue,exception);
+  if (edge_image == (Image *) NULL)
+    return(bounds);
+  (void) ParseAbsoluteGeometry("0x0+0+0",&edge_image->page);
+  memset(&vertex,0,sizeof(vertex));
+  edge_view=AcquireVirtualCacheView(edge_image,exception);
+  edge.left=GetEdgeBackgroundFactor(edge_image,edge_view,WestGravity,
+    1,0,0,0,exception);
+  edge.right=GetEdgeBackgroundFactor(edge_image,edge_view,EastGravity,
+    1,0,0,0,exception);
+  edge.top=GetEdgeBackgroundFactor(edge_image,edge_view,NorthGravity,
+    0,1,0,0,exception);
+  edge.bottom=GetEdgeBackgroundFactor(edge_image,edge_view,SouthGravity,
+    0,1,0,0,exception);
+  percent_background=1.0;
+  artifact=GetImageArtifact(edge_image,"trim:percent-background");
+  if (artifact != (const char *) NULL)
+    percent_background=StringToDouble(artifact,(char **) NULL)/100.0;
+  percent_background=MagickMin(MagickMax(1.0-percent_background,MagickEpsilon),
+    1.0);
+  background_factor=GetMinEdgeBackgroundFactor(&edge);
+  for ( ; background_factor < percent_background;
+          background_factor=GetMinEdgeBackgroundFactor(&edge))
+  {
+    if ((bounds.width == 0) || (bounds.height == 0))
+      break;
+    if (fabs(edge.left-background_factor) < MagickEpsilon)
+      {
+        /*
+          Trim left edge.
+        */
+        vertex.left++;
+        bounds.width--;
+        edge.left=GetEdgeBackgroundFactor(edge_image,edge_view,
+          NorthWestGravity,1,bounds.height,(ssize_t) vertex.left,(ssize_t)
+          vertex.top,exception);
+        edge.top=GetEdgeBackgroundFactor(edge_image,edge_view,
+          NorthWestGravity,bounds.width,1,(ssize_t) vertex.left,(ssize_t)
+          vertex.top,exception);
+        edge.bottom=GetEdgeBackgroundFactor(edge_image,edge_view,
+          SouthWestGravity,bounds.width,1,(ssize_t) vertex.left,(ssize_t)
+          vertex.bottom,exception);
+        continue;
+      }
+    if (fabs(edge.right-background_factor) < MagickEpsilon)
+      {
+        /*
+          Trim right edge.
+        */
+        vertex.right++;
+        bounds.width--;
+        edge.right=GetEdgeBackgroundFactor(edge_image,edge_view,
+          NorthEastGravity,1,bounds.height,(ssize_t) vertex.right,(ssize_t)
+          vertex.top,exception);
+        edge.top=GetEdgeBackgroundFactor(edge_image,edge_view,
+          NorthWestGravity,bounds.width,1,(ssize_t) vertex.left,(ssize_t)
+          vertex.top,exception);
+        edge.bottom=GetEdgeBackgroundFactor(edge_image,edge_view,
+          SouthWestGravity,bounds.width,1,(ssize_t) vertex.left,(ssize_t)
+          vertex.bottom,exception);
+        continue;
+      }
+    if (fabs(edge.top-background_factor) < MagickEpsilon)
+      {
+        /*
+          Trim top edge.
+        */
+        vertex.top++;
+        bounds.height--;
+        edge.left=GetEdgeBackgroundFactor(edge_image,edge_view,
+          NorthWestGravity,1,bounds.height,(ssize_t) vertex.left,(ssize_t)
+          vertex.top,exception);
+        edge.right=GetEdgeBackgroundFactor(edge_image,edge_view,
+          NorthEastGravity,1,bounds.height,(ssize_t) vertex.right,(ssize_t)
+          vertex.top,exception);
+        edge.top=GetEdgeBackgroundFactor(edge_image,edge_view,
+          NorthWestGravity,bounds.width,1,(ssize_t) vertex.left,(ssize_t)
+          vertex.top,exception);
+        continue;
+      }
+    if (fabs(edge.bottom-background_factor) < MagickEpsilon)
+      {
+        /*
+          Trim bottom edge.
+        */
+        vertex.bottom++;
+        bounds.height--;
+        edge.left=GetEdgeBackgroundFactor(edge_image,edge_view,
+          NorthWestGravity,1,bounds.height,(ssize_t) vertex.left,(ssize_t)
+          vertex.top,exception);
+        edge.right=GetEdgeBackgroundFactor(edge_image,edge_view,
+          NorthEastGravity,1,bounds.height,(ssize_t) vertex.right,(ssize_t)
+          vertex.top,exception);
+        edge.bottom=GetEdgeBackgroundFactor(edge_image,edge_view,
+          SouthWestGravity,bounds.width,1,(ssize_t) vertex.left,(ssize_t)
+          vertex.bottom,exception);
+        continue;
+      }
+  }
+  edge_view=DestroyCacheView(edge_view);
+  edge_image=DestroyImage(edge_image);
+  bounds.x=(ssize_t) vertex.left;
+  bounds.y=(ssize_t) vertex.top;
+  if ((bounds.width == 0) || (bounds.height == 0))
+    (void) ThrowMagickException(exception,GetMagickModule(),OptionWarning,
+      "GeometryDoesNotContainImage","`%s'",image->filename);
+  return(bounds);
+}
+
 MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
   ExceptionInfo *exception)
 {
   CacheView
     *image_view;
 
+  const char
+    *artifact;
+
   MagickBooleanType
     status;
 
@@ -145,9 +405,12 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
     y;
 
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  artifact=GetImageArtifact(image,"trim:percent-background");
+  if (artifact != (const char *) NULL)
+    return(GetEdgeBoundingBox(image,exception));
   bounds.width=0;
   bounds.height=0;
   bounds.x=(ssize_t) image->columns;
@@ -177,7 +440,7 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
   status=MagickTrue;
   GetMagickPixelPacket(image,&zero);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,4) shared(status) \
+  #pragma omp parallel for schedule(static) shared(status) \
     magick_number_threads(image,image,image->rows,1)
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
@@ -243,7 +506,7 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
     }
   }
   image_view=DestroyCacheView(image_view);
-  if ((bounds.width == 0) && (bounds.height == 0))
+  if ((bounds.width == 0) || (bounds.height == 0))
     (void) ThrowMagickException(exception,GetMagickModule(),OptionWarning,
       "GeometryDoesNotContainImage","`%s'",image->filename);
   else
@@ -312,7 +575,7 @@ MagickExport size_t GetImageChannelDepth(const Image *image,
   */
   assert(image != (Image *) NULL);
 
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   number_threads=(size_t) GetMagickResourceLimit(ThreadResource);
@@ -326,7 +589,7 @@ MagickExport size_t GetImageChannelDepth(const Image *image,
   if ((image->storage_class == PseudoClass) && (image->matte == MagickFalse))
     {
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-      #pragma omp parallel for schedule(static,4) shared(status) \
+      #pragma omp parallel for schedule(static) shared(status) \
         magick_number_threads(image,image,image->colors,1)
 #endif
       for (i=0; i < (ssize_t) image->colors; i++)
@@ -344,7 +607,7 @@ MagickExport size_t GetImageChannelDepth(const Image *image,
 
           atDepth=MagickTrue;
           range=GetQuantumRange(current_depth[id]);
-          if ((atDepth != MagickFalse) && ((channel & RedChannel) != 0))
+          if ((channel & RedChannel) != 0)
             if (IsPixelAtDepth(image->colormap[i].red,range) == MagickFalse)
               atDepth=MagickFalse;
           if ((atDepth != MagickFalse) && ((channel & GreenChannel) != 0))
@@ -401,7 +664,7 @@ RestoreMSCWarning
         depth_map[i]=depth;
       }
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-      #pragma omp parallel for schedule(static,4) shared(status) \
+      #pragma omp parallel for schedule(static) shared(status) \
         magick_number_threads(image,image,image->rows,1)
 #endif
       for (y=0; y < (ssize_t) image->rows; y++)
@@ -477,7 +740,7 @@ RestoreMSCWarning
     }
 #endif
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,4) shared(status) \
+  #pragma omp parallel for schedule(static) shared(status) \
     magick_number_threads(image,image,image->rows,1)
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
@@ -632,7 +895,7 @@ MagickExport size_t GetImageQuantumDepth(const Image *image,
 MagickExport ImageType GetImageType(const Image *image,ExceptionInfo *exception)
 {
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (image->colorspace == CMYKColorspace)
@@ -703,7 +966,7 @@ MagickExport ImageType IdentifyImageGray(const Image *image,
     y;
 
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if ((image->type == BilevelType) || (image->type == GrayscaleType) ||
@@ -784,7 +1047,7 @@ MagickExport MagickBooleanType IdentifyImageMonochrome(const Image *image,
     y;
 
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (image->type == BilevelType)
@@ -852,7 +1115,7 @@ MagickExport ImageType IdentifyImageType(const Image *image,
   ExceptionInfo *exception)
 {
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (image->colorspace == CMYKColorspace)
@@ -910,7 +1173,8 @@ MagickExport MagickBooleanType IsGrayImage(const Image *image,
   ExceptionInfo *exception)
 {
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
+  magick_unreferenced(exception);
   if ((image->type == BilevelType) || (image->type == GrayscaleType) ||
       (image->type == GrayscaleMatteType))
     return(MagickTrue);
@@ -946,7 +1210,8 @@ MagickExport MagickBooleanType IsMonochromeImage(const Image *image,
   ExceptionInfo *exception)
 {
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
+  magick_unreferenced(image);
   if (image->type == BilevelType)
     return(MagickTrue);
   return(MagickFalse);
@@ -997,7 +1262,7 @@ MagickExport MagickBooleanType IsOpaqueImage(const Image *image,
     Determine if image is opaque.
   */
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (image->matte == MagickFalse)
@@ -1077,7 +1342,7 @@ MagickExport MagickBooleanType SetImageChannelDepth(Image *image,
   assert(image != (Image *) NULL);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (depth >= MAGICKCORE_QUANTUM_DEPTH)
     {
       image->depth=depth;
@@ -1090,7 +1355,7 @@ MagickExport MagickBooleanType SetImageChannelDepth(Image *image,
         i;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-      #pragma omp parallel for schedule(static,4) shared(status) \
+      #pragma omp parallel for schedule(static) shared(status) \
         magick_number_threads(image,image,image->rows,1)
 #endif
       for (i=0; i < (ssize_t) image->colors; i++)
@@ -1134,7 +1399,7 @@ RestoreMSCWarning
         depth_map[i]=ScaleAnyToQuantum(ScaleQuantumToAny((Quantum) i,range),
           range);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-      #pragma omp parallel for schedule(static,4) shared(status) \
+      #pragma omp parallel for schedule(static) shared(status) \
         magick_number_threads(image,image,image->rows,1)
 #endif
       for (y=0; y < (ssize_t) image->rows; y++)
@@ -1184,7 +1449,7 @@ RestoreMSCWarning
     Scale pixels to desired depth.
   */
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,4) shared(status) \
+  #pragma omp parallel for schedule(static) shared(status) \
     magick_number_threads(image,image,image->rows,1)
 #endif
   for (y=0; y < (ssize_t) image->rows; y++)
@@ -1276,7 +1541,7 @@ MagickExport MagickBooleanType SetImageType(Image *image,const ImageType type)
   assert(image != (Image *) NULL);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   status=MagickTrue;
   image_info=AcquireImageInfo();
   image_info->dither=image->dither;
@@ -1287,38 +1552,32 @@ MagickExport MagickBooleanType SetImageType(Image *image,const ImageType type)
   {
     case BilevelType:
     {
-      if (SetImageMonochrome(image,&image->exception) == MagickFalse)
-       {
-          status=TransformImageColorspace(image,GRAYColorspace);
-          (void) NormalizeImage(image);
-          quantize_info=AcquireQuantizeInfo(image_info);
-          quantize_info->number_colors=2;
-          quantize_info->colorspace=GRAYColorspace;
-          status=QuantizeImage(quantize_info,image);
-          quantize_info=DestroyQuantizeInfo(quantize_info);
-        }
+      status=TransformImageColorspace(image,GRAYColorspace);
+      (void) NormalizeImage(image);
+      quantize_info=AcquireQuantizeInfo(image_info);
+      quantize_info->number_colors=2;
+      quantize_info->colorspace=GRAYColorspace;
+      status=QuantizeImage(quantize_info,image);
+      quantize_info=DestroyQuantizeInfo(quantize_info);
       image->matte=MagickFalse;
       break;
     }
     case GrayscaleType:
     {
-      if (SetImageGray(image,&image->exception) == MagickFalse)
-        status=TransformImageColorspace(image,GRAYColorspace);
+      status=TransformImageColorspace(image,GRAYColorspace);
       image->matte=MagickFalse;
       break;
     }
     case GrayscaleMatteType:
     {
-      if (SetImageGray(image,&image->exception) == MagickFalse)
-        status=TransformImageColorspace(image,GRAYColorspace);
+      status=TransformImageColorspace(image,GRAYColorspace);
       if (image->matte == MagickFalse)
         (void) SetImageAlphaChannel(image,OpaqueAlphaChannel);
       break;
     }
     case PaletteType:
     {
-      if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
-        status=TransformImageColorspace(image,sRGBColorspace);
+      status=TransformImageColorspace(image,sRGBColorspace);
       if ((image->storage_class == DirectClass) || (image->colors > 256))
         {
           quantize_info=AcquireQuantizeInfo(image_info);
@@ -1331,8 +1590,7 @@ MagickExport MagickBooleanType SetImageType(Image *image,const ImageType type)
     }
     case PaletteBilevelMatteType:
     {
-      if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
-        status=TransformImageColorspace(image,sRGBColorspace);
+      status=TransformImageColorspace(image,sRGBColorspace);
       if (image->matte == MagickFalse)
         (void) SetImageAlphaChannel(image,OpaqueAlphaChannel);
       (void) BilevelImageChannel(image,AlphaChannel,(double) QuantumRange/2.0);
@@ -1343,8 +1601,7 @@ MagickExport MagickBooleanType SetImageType(Image *image,const ImageType type)
     }
     case PaletteMatteType:
     {
-      if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
-        status=TransformImageColorspace(image,sRGBColorspace);
+      status=TransformImageColorspace(image,sRGBColorspace);
       if (image->matte == MagickFalse)
         (void) SetImageAlphaChannel(image,OpaqueAlphaChannel);
       quantize_info=AcquireQuantizeInfo(image_info);
@@ -1355,8 +1612,7 @@ MagickExport MagickBooleanType SetImageType(Image *image,const ImageType type)
     }
     case TrueColorType:
     {
-      if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
-        status=TransformImageColorspace(image,sRGBColorspace);
+      status=TransformImageColorspace(image,sRGBColorspace);
       if (image->storage_class != DirectClass)
         status=SetImageStorageClass(image,DirectClass);
       image->matte=MagickFalse;
@@ -1364,8 +1620,7 @@ MagickExport MagickBooleanType SetImageType(Image *image,const ImageType type)
     }
     case TrueColorMatteType:
     {
-      if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
-        status=TransformImageColorspace(image,sRGBColorspace);
+      status=TransformImageColorspace(image,sRGBColorspace);
       if (image->storage_class != DirectClass)
         status=SetImageStorageClass(image,DirectClass);
       if (image->matte == MagickFalse)
@@ -1374,12 +1629,7 @@ MagickExport MagickBooleanType SetImageType(Image *image,const ImageType type)
     }
     case ColorSeparationType:
     {
-      if (image->colorspace != CMYKColorspace)
-        {
-          if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
-            (void) TransformImageColorspace(image,sRGBColorspace);
-          status=TransformImageColorspace(image,CMYKColorspace);
-        }
+      status=TransformImageColorspace(image,CMYKColorspace);
       if (image->storage_class != DirectClass)
         status=SetImageStorageClass(image,DirectClass);
       image->matte=MagickFalse;
@@ -1387,12 +1637,7 @@ MagickExport MagickBooleanType SetImageType(Image *image,const ImageType type)
     }
     case ColorSeparationMatteType:
     {
-      if (image->colorspace != CMYKColorspace)
-        {
-          if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
-            (void) TransformImageColorspace(image,sRGBColorspace);
-          status=TransformImageColorspace(image,CMYKColorspace);
-        }
+      status=TransformImageColorspace(image,CMYKColorspace);
       if (image->storage_class != DirectClass)
         status=SetImageStorageClass(image,DirectClass);
       if (image->matte == MagickFalse)
